/*
   accsctrl.c

   Handlers for access control commands.

   Copyright (C) 1999   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (libraftp@narod.ru)


   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/
/* $Id: accsctrl.c,v 1.8 2008/04/30 23:14:20 shesmax Exp $ */

#include <malloc.h>
#include <string.h>

#include "libra.h"
#include "pi.h"
#include "dtp.h"
#include "parser.h"
#include "retcodes.h"
#include "log.h"

/*
* FTP command handlers
*/

static char sLoggedIn[] = "230 Logged in, no need for password\r\n";

/* Superfluous command for an anonymous site */
int User( char* unused UNUSED_PARAM )
{
   return SendReply( sLoggedIn );
}

/* Superfluous command for an anonymous site */
int Pass( char* unused UNUSED_PARAM )
{
   return SendReply( sLoggedIn );
}

/* Specifies the user's account. */
int Acct( char* args UNUSED_PARAM )
{
   return SendReply
   (
      "202 Command not implemented, superfluous at this site\r\n"
   );
}

/* Mount different file system data structure. */
int Smnt( char* args UNUSED_PARAM )
{
   return SendReply( "502 Command not implemented\r\n" );
}

/* Reset all parameters to default settings. */
int Rein( char* args UNUSED_PARAM )
{
   ResetUTF8Usage();
   ShutdownDTP();
   if( InitDTP() )
      return SendReply( "220 Service ready\r\n" );
   else
   {
      SendReply( "421 Cannot restart the data transfer process. Aborting\r\n" );
      return RC_ABORTED;
   }
}

/* terminate USER, close control connection. */
int Quit( char* args UNUSED_PARAM )
{
   /* Do not check whether the reply is delivered to the remote host. */
   SendReply( "221 Bye\r\n" );
   return RC_ABORTED;
}

