/*
   dataopen.h   

   Miscellaneous DTP declarations.

   Copyright (C) 1999   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (libraftp@narod.ru)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/
/* $Id: dataopen.h,v 1.9 2002/05/18 05:50:02 shesmax Exp $ */

#ifndef __GUARD_DATAOPEN_H
#define __GUARD_DATAOPEN_H

#include "libra.h"

/*
** DataOpen imports
*/

extern int hDataSocket; /* Socket for data connection. It should be */
                        /* public because its life scope is more    */
                        /* then one command (f.e. passive mode).    */
#include "dtp.h"

extern nb_t nDtPassed;  /* current tranfer position */
extern nb_t nDtTotal;   /* total file size for RETR */
extern nb_t nRestartAt; /* restart position */

/*
** DataOpen exports
*/

/* Creates socket for the data connection. */
int CreateDataSocket( int *hDataSocket );

/*
   Opens either passive or active data connection. Returns RC_TRUE if the
   data connection was opened.
*/
int OpenDataConnection( int *hDataSocket );

/* Closes data connection. */
void CloseDataConnection( int *hDataSocket );

#include <sys/stat.h>

#define INVALID_FILE_DESCRIPTOR (-1)
int OpenFile( const char* szPath, int flags, mode_t mode,
      struct stat *st );

void LogTransferStat(const char *szPath, struct timeval *pstart,
      const char *szLabel, const char *szStatus);

/*
** Data operations import
*/

extern int nMaxMap;                    /* Maximum mapping length */

/*
** Data operations export
*/

int DoRetrieve( int* hDataSocket, const char* szPath );
int DoStore( int* hDataSocket, const char* szPath, int fd, int flags );

#endif
