/*
   dtp.h

   Data Transfer Process

   Copyright (C) 1999   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (libraftp@narod.ru)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/
/* $Id: dtp.h,v 1.14 2002/05/18 05:50:02 shesmax Exp $ */

#ifndef __GUARD_DTP_H
#define __GUARD_DTP_H

/* Initialize the DTP. Returns RC_TRUE if case of successfull
 * initialization, otherwise returns RC_FALSE */
int  InitDTP();

/* Closes data connection (if any) */
void ShutdownDTP();

/*
*   FTP command handlers
*/

int Appe( char* args );
int List( char* args );
int Mode( char* args );
int Nlst( char* args );
int Pasv( char* args );
int Port( char* args );
int EPsv( char* args );
int EPrt( char* args );
int Rest( char* args );
int Retr( char* args );
int Stor( char* args );
int Stou( char* args );
int Stru( char* args );
int Type( char* args );

/*
*  Query and control DTP
*/

/* Transfer mode */
#define TRANSFER_MODE_STREAM     'S'
#define TRANSFER_MODE_BLOCK      'B'
#define TRANSFER_MODE_COMPRESSED 'C'

/* Dataset structure */
#define STRUCTURE_FILE           'F'
#define STRUCTURE_RECORDS        'R'
#define STRUCTURE_PAGES          'P'

/* Representation type */
#define TYPE_ASCII               'A'
#define TYPE_EBCDIC              'E'
#define TYPE_IMAGE               'I'
#define TYPE_LOCAL               'L'

/* Formats */
#define FORMAT_NONPRINT          'N'
#define FORMAT_TELNET            'T'
#define FORMAT_ASA               'A'

char* GetRepresentationType();
char* GetFileStructure();
char* GetTransferMode();
char* GetDataConnType();

/* Returns RC_TRUE if DTP is sending data through data connection. */
int DataTransferInProgress();

/* Set TransferInProgress mark. */
void StartTransfer();

/* Reset TransferInProgress mark. */
void StopTransfer();

/* Returns RC_TRUE if the data transfer should be aborted. */
int AbortTransfer();

/* Posts the message to DTP to abort the transfer of data. There is some
 *	delay between the message is posted and the transfer is aborted. */
void PostTransferAbort();

/* Cancels abort of data transfer */
void CancelTransferAbort();

/* Type that holds the number of bytes */
typedef unsigned long nb_t;

/* Returns the number of bytes sent so far */
nb_t DataSent();

/* Returns the number of bytes should be sent */
nb_t DataToSend();

/* Returns RC_TRUE if the server is started in "passive data connections 
 * only" mode (command line option -p). Otherwise returns RC_FALSE. */
int PassiveOnlyMode();

/* Returns RC_TRUE if peer requested server to accept EPSV command only. At
 * some time a peer can send us EPSV ALL indicating that in the future it will
 * use EPSV only. After this time server should reject all PORT, PASV and EPRT
 * commands. There is no way to turn this flag off. */
int EpsvOnly();

/* Sets "passive data connections only" mode. */
void SetPassiveOnlyMode();

extern int bPassive;    /* 1 = passive connection */

/*
*  Data connection addresses handling
*/

#include "sockets.h"

#define DEFAULT_DATA_PORT  21

extern PSaddr addrPeerData; /* Peer */
extern PSaddr addrSrvrData; /* Our host */

/* Sets address of server's data connection end point */
int SrvrDataConn( sa_family_t af );

/*
*  Checksums (MD5)
*/

/* Returns RC_TRUE if the checksums are computed for files to be sent */
int ComputeChecksums();

/* If on is equal to RC_TRUE, the program will compute the checksum of 
 *	the file to be sent. It will send this sum in the 226 reply through
 *	the control connection to the peer.  */
void SetChecksumMode( int on );

/*
*  Helper routines for LIST, NLST, MLSD commands 
*/

#include "xbuf.h"
#include "listdir.h"

struct DataConnectionListingParams
{
   XBuf* xb;      /* Buffers output to data connection */
   int   socket;  /* Data connection socket */
   void* params;  /* User-supplied parameters */
};
typedef struct DataConnectionListingParams DCLP;

int DtpLs( char* path, LsCallback cb, void* params );

#endif
