/*
 	file.c

	Incorporates handlers for commands dealing with files:
      RNFR -> RNTO, DELE, SIZE, MDTM.

   Copyright (C) 1999   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (libraftp@narod.ru)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/
/* $Id: file.c,v 1.9 2002/12/14 23:16:36 shesmax Exp $ */

#include <unistd.h>
#include <time.h>
#include <sys/stat.h>
#include <errno.h>

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "libra.h"
#include "pi.h"
#include "dtp.h"
#include "parser.h"
#include "log.h"

/* Holds the name passed in RNFR command. */
static char* szRnFr = 0;

/* Specifies the old pathname in RNFR-RNTO sequence. */
int Rnfr( char* args )
{
	char* szPath;
	
	if( !szRnFr )		/* We have got a RNFR before. We stored old name. Now we can drop it */
	{
		free( szRnFr );
		szRnFr = 0;
	}

	if( GetPath( &args, &szPath, GP_OBLIGATORY ) )
		if( ReadOnlyBehavior() )
			AccumReply( "550 No permissions\r\n" );
   	else if( access( szPath, F_OK ) == -1 )
         PError( 550 );
      else if( !(szRnFr = strdup( szPath )) )
		   PError( 450 );
      else
         AccumReply( "350 Proceed with RNTO <new name>\r\n" );
   return SendReplies();
}

/* Specifies the new pathname of the file and renames it. */
int Rnto( char*  args )
{
	char* szPath;

	if( !TestPrevCommand( "RNFR", "RNTO" ) )
      ;
   else if( !szRnFr )
      AccumReply( "553 RNFR was denied\r\n" );
	else if( GetPath( &args, &szPath, GP_OBLIGATORY ) )
      if( rename( szRnFr, szPath ) == -1 )
			PError( 553 );
		else
		{
			LogEvent( LLOG_AUDIT, "RnTo: %s -> %s\n", szRnFr, szPath );
			AccumReply( "250 File was renamed\r\n" );
		}

	/* Cleanup */
	if( szRnFr )
	{
		free( szRnFr );
		szRnFr = 0;
	}

	return SendReplies();
}

/* Delete the file specified in the pathname at the server side. */
int Dele( char* args )
{
	char* szPath;

	if( GetPath( &args, &szPath, GP_OBLIGATORY ) )
		if( ReadOnlyBehavior() )
			AccumReply( "550 No permissions\r\n" );
		else if( unlink( szPath ) == 0 )
		{
			LogEvent( LLOG_AUDIT, "Dele: %s\n", szPath );
			AccumReply( "250 File was successfully deleted\r\n" );
		}
		else
      	PError( 550 );
	return SendReplies();
}

/* Returns the size of the specified file to the user. */
int Size( char* args )
{
   char* szPath;
   struct stat st;

   if( GetPath( &args, &szPath, GP_OBLIGATORY ) )
      if( *GetRepresentationType() == TYPE_ASCII )
         AccumReply( "550 Size value is available only for types I or L\r\n" );
      else if( stat( szPath, &st ) == 0 )
         AccumReplyF( "213 %ld\r\n", st.st_size );
      else
         PError( 550 );
   return SendReplies();
}

/* Returns the modification time of the specified file. */
int Mdtm( char* args )
{
   char* szPath;
   struct stat st;

   if( GetPath( &args, &szPath, GP_OBLIGATORY ) )
      if( stat( szPath, &st ) == 0 )
      {
         struct tm* t = gmtime( (time_t*)&st.st_mtime );

         AccumReplyF
         ( 
            "213 %04d%02d%02d%02d%02d%02d\r\n",
            t->tm_year + 1900, t->tm_mon + 1, t->tm_mday,
            t->tm_hour, t->tm_min, t->tm_sec
         );
      }
      else
         PError( 550 );
   return SendReplies();
}

