/* 
	Libra.c

	Main function for the FTP daemon.

   Copyright (C) 1999   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (libraftp@narod.ru)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/

/* $Id: libra.c,v 1.23 2002/12/15 23:17:16 shesmax Exp $ */

#include <unistd.h>
#include <sys/types.h>

#include <stdlib.h>

#include "libra.h"
#include "pi.h"
#include "dtp.h"
#include "mlsx.h"
#include "utf8.h"
#include "sockets.h"
#include "retcodes.h"
#include "log.h"

static char sPgmName[] = "Libra FTP";

static int bReadOnly = RC_FALSE;
static int bBackend = RC_FALSE;

int ReadOnlyBehavior()
{
	return bReadOnly;
}

int Backend()
{
   return bBackend;
}

/*
*	Startup code
*/

static int ParseCmdLine( int argc, char* argv[] )
{
	int n;

	/* 
		If the daemon is started by inetd, it has no program name in the
		command line.

      Well, it's an inetd bug.  We shall not try to fix it _here_.
      Fortunately, tcpd is a good workaround.  --SAW
	*/
	if( !argc )
		return RC_TRUE;

   ++argv;
	while( --argc > 0 )
	{
		switch( **argv )
		{
		case '-':
			switch( (*argv)[ 1 ] )
			{
         case 'b':
            bBackend = RC_TRUE;
            break;
			case 'r':
				bReadOnly = RC_TRUE;
				break;
        case 'p':
            SetPassiveOnlyMode();
				break;

			case 'l':
				if( (n = ConfigureLog( *argv + 2 )) != 0 )
				{
					LogEvent( LLOG_ERROR, "Invalid option -l%c\n", *argv[ n ] );
					return RC_FALSE;
				}
				break;

         case 'C':
            if( utf8_read_local_to_unicode( *argv + 2 ) != RC_OK )
            {
               LogEvent( LLOG_ERROR, "Cannot read conversion table for character set\n" );
               return RC_FALSE;
            }
            break;
            
			default:
				LogEvent( LLOG_ERROR, "Invalid option %s\n", *argv );
				return RC_FALSE;
			}
			break;

		default:
			LogEvent( LLOG_ERROR, "Invalid parameter %s\n", *argv );
			return RC_FALSE;
		}
		++argv;
	}
	
	return RC_TRUE;
}

int main( int argc, char* argv[] )
{
	if( !OpenLog( sPgmName ) )
		return RC_ERROR;

	if( ParseCmdLine( argc, argv ) )
		if( InitPI() )
		{
		   if( InitDTP() )
			{
				RunPI();
				ShutdownDTP();
			}
			ShutdownPI();
		}

	CloseLog();
	return RC_OK;
}
