/* 
	listdir.c

	Enumeration function for file system entries. Calls the callback
	function for every entry that match to the given name pattern

   Copyright (C) 1999   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (libraftp@narod.ru)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/
/* $Id: listdir.c,v 1.10 2002/05/18 05:50:02 shesmax Exp $ */

#include <unistd.h>
#include <sys/stat.h>
#include <dirent.h>
#include <errno.h>

#include <stdlib.h>  /* PATH_MAX */
#include <string.h>

#include "listdir.h"
#include "dataopen.h"
#include "retcodes.h"
#include "xbuf.h"

int Ls( char* sPath, LsCallback pfCallback, void* param )
{
   DIR* d = 0;
   struct dirent* e;
   struct stat st;
   int nRetVal;
   XBuf xb;
   int nDirNameLen;
   
   /* Check the passed parameters */
   if( !pfCallback )
      return RC_ERROR;
   if( !sPath || !*sPath )
      sPath = ".";
   /* strip away the trailing slash for the sake of compatibility */
   nDirNameLen = strlen( sPath );
   if( sPath[ nDirNameLen - 1 ] == '*' )       
      sPath[ nDirNameLen - 1 ] = '.';

   /* Determine what to list - directory or some other file system object */
   if( stat( sPath, &st ) == -1 )
      return RC_ERROR;
         
   if( !S_ISDIR( st.st_mode ) )       
      return lstat( sPath, &st ) == -1 ? 
         RC_ERROR : pfCallback( sPath, &st, param );

   /* List directory */
   
	if( CreateExpandableXBuf( &xb, PATH_MAX ) == XBUF_OVERFLOW )
		return RC_ERROR;
   nRetVal = RC_ERROR;
   do
   {
      if( !(*sPath == '.' && !sPath[ 1 ]) )
      {
         nDirNameLen = strlen( sPath );
         if( WriteXBuf( &xb, sPath, nDirNameLen ) == XBUF_OVERFLOW )
            break;
         if( sPath[ nDirNameLen - 1 ] != '/' )
         {
            if( WriteXBuf( &xb, "/", 1 ) == XBUF_OVERFLOW )
               break;
            ++nDirNameLen;
         }
      }
      else
         nDirNameLen = 0;

      if( (d = opendir( sPath )) == 0 )
         break;
      nRetVal = RC_OK;
      while( (e = readdir( d )) != 0 )
      {
         if
         ( 
            e->d_name[ 0 ] == '.' && 
            (!e->d_name[ 1 ] || (e->d_name[ 1 ] == '.' && !e->d_name[ 2 ])) 
         )   
            continue; 
         xb.nUsed = nDirNameLen; /* restrict buffer to directory name */
         if
         ( 
            WriteXBuf(&xb, e->d_name, strlen(e->d_name)+1) == XBUF_OVERFLOW ||
            lstat( xb.data, &st ) == -1 
         )
         {
            nRetVal = RC_ERROR;
            break;
         }
         nRetVal = pfCallback( xb.data, &st, param );
         if( nRetVal != RC_OK )
            break;
      }
   }
   while( 0 );

   if( d )
      closedir( d );
   DestroyXBuf( &xb );
   return nRetVal;
}
