/* 
	log.h

	Log facility. Writes log messages to stderr.

   Copyright (C) 1999   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (libraftp@narod.ru)

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/
/* $Id: log.h,v 1.10 2002/05/18 05:50:02 shesmax Exp $ */

#ifndef __GUARD_LOG_H
#define __GUARD_LOG_H

/* Internal constants for logging events */
#define _LLOG_ERROR		0x01
#define _LLOG_WARN	   0x02
#define _LLOG_CONNECT   0x04
#define _LLOG_AUDIT		0x08
#define _LLOG_CMD			0x10
#define _LLOG_REPLY		0x20
#define LOG_EVENT( event )  ((event) & 0xFF)

/* Log event severity */
#define _LLOG_DONTCARE	0x1000	/* if we fail to log, do nothing */
#define _LLOG_FATAL		0x2000	/* if we fail to log, end session */
#define LOG_SEVERITY( event ) (((event) & 0xFF00) >> 8)

/* Logging events (for external use) */
#define LLOG_ERROR	(_LLOG_ERROR	| _LLOG_FATAL)
#define LLOG_WARN		(_LLOG_WARN 	| _LLOG_DONTCARE)
#define LLOG_CONNECT	(_LLOG_CONNECT | _LLOG_DONTCARE)	
#define LLOG_AUDIT	(_LLOG_AUDIT 	| _LLOG_DONTCARE)
#define LLOG_CMD		(_LLOG_CMD		| _LLOG_DONTCARE)
#define LLOG_REPLY   (_LLOG_REPLY   | _LLOG_DONTCARE)

/* Function prototypes */

/* Enables log facility. Returns either RC_TRUE or RC_FALSE */
int OpenLog( const char* sPgmName );
/* Disables log facility. Returns either RC_TRUE or RC_FALSE */
int CloseLog();
/* 
	Sets the mask of events to be logged. See ftps.man for details. 
	Returns RC_TRUE, if parameter(s) passed to it are correct.
	Otherwise returns RC_FALSE.
*/
int ConfigureLog( const char* params );

/* 
	Logs an event. If this event is not in the mask of events to
	be logged this call has no effects, otherwise the function writes
   the given message to the log.

	event		event to be logged
	fmt		caller defined message.

	Returns RC_OK or RC_ERROR.
*/	
int LogEvent( int event, const char* fmt, ... );

#endif
