/*
   utf8.h

   Routines dealing with UTF-8 encoding of pathnames sent over control connection.

   Copyright (C) 2002   Andrey Savochkin (saw@saw.sw.com.sg)
                        Maxim Shesterikov (libraftp@narod.ru)
                 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA  02111-1307, USA.
*/

/* $Id: utf8.h,v 1.4 2002/12/15 23:17:16 shesmax Exp $ */

#ifndef __GUARD_UTF8_H
#define __GUARD_UTF8_H

#include <stdlib.h>

/* Determines encoding type of string buf. Returns one of the following
 * constants. */
#define ENCTYPE_ASCII      0     // buf contains 7-bit ASCII string
#define ENCTYPE_LOCAL      1     // buf contains 8-bit string
#define ENCTYPE_UTF8       2     // buf contains UTF8 string
int utf8_enctype( const unsigned char *buf, unsigned int len );

/* Conversion between local encoding and UTF-8. Returns length of result
 * string (including \0). Returns 0 if there was no memory to allocate
 * function conversion buffers */
size_t utf8_to_local( const unsigned char *utf8, size_t len, unsigned char **local );
size_t local_to_utf8( const unsigned char *local, size_t len, unsigned char **utf8 );

// Loads conversion table between local encoding and UNICODE. Returns RC_OK
// if everything was ok, otherwise returns RC_ERROR;
int utf8_read_local_to_unicode( const char* file );

#endif
